# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

module Selenium
  module WebDriver
    module Support
      class Guards
        #
        # Guard derived from RSpec example metadata.
        # @api private
        #

        class GuardCondition
          attr_accessor :name, :execution

          def initialize(name, condition = nil, &blk)
            @name = name
            @execution = if blk
                           proc(&blk)
                         else
                           proc { |guarded| guarded.include?(condition) }
                         end
          end

          def satisfied?(guard)
            list = Array(guard.guarded[@name])

            list.empty? || @execution.call(list)
          end
        end # GuardCondition
      end # Guards
    end # Support
  end # WebDriver
end # Selenium
