# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# This file is automatically generated. DO NOT EDIT!
# Regenerate with: bazel run //rb/lib/selenium/webdriver:bidi-generate

module Selenium
  module WebDriver
    class BiDi
      module Protocol
        # @api private
        # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
        class Speculation < Domain
          EVENTS = {
            prefetch_status_updated: 'speculation.prefetchStatusUpdated'
          }.freeze

          PRELOADING_STATUS = {
            pending: 'pending',
            ready: 'ready',
            success: 'success',
            failure: 'failure'
          }.freeze

          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          PrefetchStatusUpdatedParameters = Serialization::Record.define(
            context: {wire_key: 'context', primitive: 'string'},
            url: {wire_key: 'url', primitive: 'string'},
            status: {wire_key: 'status', enum: 'Speculation::PRELOADING_STATUS'}
          )

          EVENT_TYPES = {
            'speculation.prefetchStatusUpdated' => Speculation::PrefetchStatusUpdatedParameters
          }.freeze
        end # Speculation
      end # Protocol
    end # BiDi
  end # WebDriver
end # Selenium
