# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

module Selenium
  module WebDriver
    module Safari
      class Options < WebDriver::Options
        attr_accessor :options

        # @see https://developer.apple.com/documentation/webkit/about_webdriver_for_safari
        CAPABILITIES = {automatic_inspection: 'safari:automaticInspection',
                        automatic_profiling: 'safari:automaticProfiling'}.freeze
        BROWSER = 'safari'
        TECHNOLOGY_PREVIEW = 'Safari Technology Preview'

        def add_option(name, value = nil)
          key = name.is_a?(Hash) ? name.keys.first : name
          raise ArgumentError, 'Safari does not support options that are not namespaced' unless key.to_s.include?(':')

          super
        end

        def browser_name=(value)
          @options[:browser_name] = value
        end

        def browser_name
          @options[:browser_name] = Safari.technology_preview? ? TECHNOLOGY_PREVIEW : BROWSER
        end
      end # Options
    end # Safari
  end # WebDriver
end # Selenium
