# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# This file is automatically generated. DO NOT EDIT!
# Regenerate with: bazel run //rb/lib/selenium/webdriver:bidi-generate

module Selenium
  module WebDriver
    class BiDi
      module Protocol
        # @api private
        # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
        class WebExtension < Domain
          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          InstallParameters = Serialization::Record.define(
            extension_data: {wire_key: 'extensionData', ref: 'WebExtension::ExtensionData'}
          )

          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          class ExtensionData < Serialization::Union
            discriminator 'type', {archive_path: 'archivePath', base64: 'base64', path: 'path'}
            variants(
              archive_path: 'WebExtension::ExtensionArchivePath',
              base64: 'WebExtension::ExtensionBase64Encoded',
              path: 'WebExtension::ExtensionPath'
            )
          end

          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          ExtensionPath = Serialization::Record.define(
            type: {fixed: 'path'},
            path: {wire_key: 'path', primitive: 'string'}
          )

          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          ExtensionArchivePath = Serialization::Record.define(
            type: {fixed: 'archivePath'},
            path: {wire_key: 'path', primitive: 'string'}
          )

          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          ExtensionBase64Encoded = Serialization::Record.define(
            type: {fixed: 'base64'},
            value: {wire_key: 'value', primitive: 'string'}
          )

          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          InstallResult = Serialization::Record.define(extension: 'extension')

          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          UninstallParameters = Serialization::Record.define(extension: 'extension')

          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          def install(extension_data:)
            params = InstallParameters.new(extension_data: extension_data)
            execute(cmd: 'webExtension.install', params: params, result: WebExtension::InstallResult)
          end

          # @api private
          # @see https://www.selenium.dev/documentation/warnings/bidi-implementation/
          def uninstall(extension:)
            params = UninstallParameters.new(extension: extension)
            execute(cmd: 'webExtension.uninstall', params: params)
          end
        end # WebExtension
      end # Protocol
    end # BiDi
  end # WebDriver
end # Selenium
