# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

require 'uri'
require 'selenium/webdriver/remote/server_error'

module Selenium
  module WebDriver
    module Remote
      autoload :Features,     'selenium/webdriver/remote/features'
      autoload :Bridge,       'selenium/webdriver/remote/bridge'
      autoload :BiDiBridge,   'selenium/webdriver/remote/bidi_bridge'
      autoload :Driver,       'selenium/webdriver/remote/driver'
      autoload :Response,     'selenium/webdriver/remote/response'
      autoload :Capabilities, 'selenium/webdriver/remote/capabilities'

      module Http
        autoload :Common, 'selenium/webdriver/remote/http/common'
        autoload :Default, 'selenium/webdriver/remote/http/default'
      end
    end
  end
end
