# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

module Selenium
  module WebDriver
    module DriverExtensions
      module HasNetworkInterception
        #
        # Intercepts requests coming from browser allowing
        # to either pass them through like proxy or provide
        # a stubbed response instead.
        #
        # @example Log requests and pass through
        #   driver.intercept do |request, &continue|
        #     puts "#{request.method} #{request.url}"
        #     continue.call(request)
        #   end
        #
        # @example Stub requests for images
        #   driver.intercept do |request, &continue|
        #     if request.url.match?(/\.png$/)
        #       request.url = 'https://upload.wikimedia.org/wikipedia/commons/d/d5/Selenium_Logo.png'
        #     end
        #     continue.call(request)
        #   end
        #
        # @example Log responses and pass through
        #   driver.intercept do |request, &continue|
        #     continue.call(request) do |response|
        #       puts "#{response.code} #{response.body}"
        #     end
        #   end
        #
        # @example Mutate specific response
        #   driver.intercept do |request, &continue|
        #     continue.call(request) do |response|
        #       response.body << 'Added by Selenium!' if request.url.include?('/myurl')
        #     end
        #   end
        #
        # @param [Proc] block which is called when request is intercepted
        # @yieldparam [DevTools::Request] request
        # @yieldparam [Proc] continue block which proceeds with the request and optionally yields response
        #

        def intercept(&block)
          if browser == :firefox
            WebDriver.logger.deprecate(
              'Driver#intercept on Firefox',
              'the new bidi.network.add_intercept method',
              id: :intercept
            )
          end
          @interceptor ||= DevTools::NetworkInterceptor.new(devtools)
          @interceptor.intercept(&block)
        end
      end # HasNetworkInterception
    end # DriverExtensions
  end # WebDriver
end # Selenium
