# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

module Selenium
  module WebDriver
    class BiDi
      # Implements the Session Module of the WebDriver-BiDi specification
      #
      # @api private
      #
      class Session
        Status = Struct.new(:ready, :message)

        def initialize(bidi)
          @bidi = bidi
        end

        def status
          status = @bidi.send_cmd('session.status')
          Status.new(**status)
        end

        def subscribe(events, browsing_contexts = nil)
          opts = {events: Array(events)}
          opts[:browsing_contexts] = Array(browsing_contexts) if browsing_contexts

          @bidi.send_cmd('session.subscribe', **opts)
        end

        def unsubscribe(events, browsing_contexts = nil)
          opts = {events: Array(events)}
          opts[:browsing_contexts] = Array(browsing_contexts) if browsing_contexts

          @bidi.send_cmd('session.unsubscribe', **opts)
        end
      end # Session
    end # BiDi
  end # WebDriver
end # Selenium
