# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

module Selenium
  module WebDriver
    module IE
      class Service < WebDriver::Service
        DEFAULT_PORT = 5555
        EXECUTABLE = 'IEDriverServer'
        SHUTDOWN_SUPPORTED = true
        DRIVER_PATH_ENV_KEY = 'SE_IEDRIVER'

        def initialize(args: nil, **)
          if ENV.key?('SE_DEBUG')
            args = Array(args.dup)
            warn_driver_log_override if args.reject! { |arg| arg.include?('log-level') || arg.include?('silent') }
            args << '--log-level=DEBUG'
          end

          super
        end
      end # Server
    end # IE
  end # WebDriver
end # Selenium
