# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

module Selenium
  module WebDriver
    module DriverExtensions
      module HasContext
        #
        # Sets the context that Selenium commands are running in using
        #         a `with` statement. The state of the context on the server is
        #         saved before entering the block, and restored upon exiting it.
        #
        # @param [String] value which context gets set (either 'chrome' or 'content')
        #

        def context=(value)
          @bridge.context = value
        end

        def context
          @bridge.context
        end
      end # HasContext
    end # DriverExtensions
  end # WebDriver
end # Selenium
