# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

module Selenium
  module WebDriver
    module Firefox
      #
      # Driver implementation for Firefox using GeckoDriver.
      # @api private
      #

      class Driver < WebDriver::Driver
        EXTENSIONS = [DriverExtensions::HasAddons,
                      DriverExtensions::FullPageScreenshot,
                      DriverExtensions::HasContext,
                      DriverExtensions::HasBiDi,
                      DriverExtensions::HasLogEvents,
                      DriverExtensions::HasNetworkInterception,
                      DriverExtensions::PrintsPage].freeze

        include LocalDriver

        def initialize(options: nil, service: nil, url: nil, http_client: nil, client_config: nil, **)
          initialize_local_driver(options, service, url, http_client, client_config) do |caps, client|
            super(caps: caps, http_client: client, **)
          end
        end

        def browser
          :firefox
        end
      end # Driver
    end # Firefox
  end # WebDriver
end # Selenium
