# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

module Selenium
  module WebDriver
    module Remote
      module Http
        class Common
          MAX_REDIRECTS   = 20 # same as chromium/gecko
          CONTENT_TYPE    = 'application/json'
          DEFAULT_HEADERS = {
            'Accept' => CONTENT_TYPE,
            'Content-Type' => "#{CONTENT_TYPE}; charset=UTF-8"
          }.freeze
          BINARY_ENCODINGS = [Encoding::BINARY, Encoding::ASCII_8BIT].freeze

          class << self
            def user_agent
              ClientConfig.default_user_agent
            end

            def user_agent=(value)
              ClientConfig.default_user_agent = value
            end

            def extra_headers
              ClientConfig.default_extra_headers
            end

            def extra_headers=(value)
              ClientConfig.default_extra_headers = value
            end
          end

          attr_reader :client_config

          def initialize(client_config: nil)
            @client_config = client_config || ClientConfig.new
          end

          def server_url=(url)
            client_config.server_url = url
          end

          def quit_errors
            [IOError]
          end

          def close
            # hook for subclasses - will be called on Driver#quit
          end

          # steep:ignore:start
          def call(verb, url, command_hash)
            url      = server_url.merge(url) unless url.is_a?(URI)
            headers  = common_headers.dup
            headers['Cache-Control'] = 'no-cache' if verb == :get

            if command_hash
              command_hash              = ensure_utf8_encoding(command_hash)
              payload                   = JSON.generate(command_hash)
              headers['Content-Length'] = payload.bytesize.to_s if %i[post put].include?(verb)

              WebDriver.logger.debug("   >>> #{url} | #{payload}", id: :command)
              WebDriver.logger.debug("     > #{headers.inspect}", id: :header)
            elsif verb == :post
              payload = '{}'
              headers['Content-Length'] = '2'
            end

            request verb, url, headers, payload
          end
          # steep:ignore:end

          private

          def common_headers
            @common_headers ||= begin
              headers = DEFAULT_HEADERS.dup
              headers['User-Agent'] = client_config.user_agent
              headers.merge(client_config.extra_headers || {})
            end
          end

          def server_url
            client_config.server_url || raise(Error::WebDriverError, 'server_url not set')
          end

          def request(*)
            raise NotImplementedError, 'subclass responsibility'
          end

          def ensure_utf8_encoding(obj)
            case obj
            when String
              encode_string_to_utf8(obj)
            when Array
              obj.map { |item| ensure_utf8_encoding(item) }
            when Hash
              obj.each_with_object({}) do |(key, value), result|
                result[ensure_utf8_encoding(key)] = ensure_utf8_encoding(value)
              end
            else
              obj
            end
          end

          def encode_string_to_utf8(str)
            return str if str.encoding == Encoding::UTF_8 && str.valid_encoding?

            if BINARY_ENCODINGS.include?(str.encoding)
              result = str.dup.force_encoding(Encoding::UTF_8)
              return result if result.valid_encoding?
            end

            str.encode(Encoding::UTF_8)
          rescue EncodingError => e
            raise Error::WebDriverError,
                  "Unable to encode string to UTF-8: #{e.message}. " \
                  "String encoding: #{str.encoding}, content: #{str.inspect}"
          end

          def create_response(code, body, content_type)
            code = code.to_i
            body = body.to_s.strip
            content_type = content_type.to_s
            WebDriver.logger.debug("<- #{body}", id: :command)

            if content_type.include? CONTENT_TYPE
              raise Error::WebDriverError, "empty body: #{content_type.inspect} (#{code})\n#{body}" if body.empty?

              Response.new(code, JSON.parse(body))
            elsif code == 204
              Response.new(code)
            else
              msg = if body.empty?
                      "unexpected response, code=#{code}, content-type=#{content_type.inspect}"
                    else
                      "unexpected response, code=#{code}, content-type=#{content_type.inspect}\n#{body}"
                    end

              raise Error::WebDriverError, msg
            end
          end
        end # Common
      end # Http
    end # Remote
  end # WebDriver
end # Selenium
